require 'rubyXL'
class Admin::JournalPapersController < OrbitMemberController
  include Admin::JournalPapersHelper
  layout "member_plugin"

  before_action :set_journal_paper, only: [:show, :edit , :update, :destroy]
  before_action :set_plugin
  before_action :get_settings,:only => [:new, :edit, :setting]

  before_action :need_access_right
  before_action :allow_admin_only, :only => [:index, :setting]

  def index
    @writing_journals = JournalPaper.order_by(:year=>'desc').page(params[:page]).per(10)
  end

  def new
    @member = MemberProfile.find_by(:uid=>params['uid']) rescue nil
    @journal_paper = JournalPaper.new

    if params[:desktop]
      render :layout => false
    end
  end

  def analysis
  end

  def analysis_report
    role = params[:role_id]
    year_start = params[:year_start].to_i
    year_end = params[:year_end].to_i
    graph_by = params[:graph_by]

    @data = get_chart_data(year_start,year_end,role,params[:graph_by])

    render :layout => false
  end

  def download_excel
    year_start = params[:year_start].to_i
    year_end = params[:year_end].to_i
    @data = get_data_for_excel(year_start,year_end)
    respond_to do |format|
      format.xlsx {
        response.headers['Content-Disposition'] = 'attachment; filename="journal_papers.xlsx"'
      }
    end
  end

  def create

    if !journal_paper_params['member_profile_id'].blank?

      @member = MemberProfile.find(journal_paper_params['member_profile_id']) rescue nil
      @journal_paper = JournalPaper.new(journal_paper_params)
      @journal_paper.save

      if params[:desktop] == "true"
        render json: {"data" => get_paper_list}.to_json
      else
        redirect_to params['referer_url']
      end

    elsif !params[:author_members].blank?

      params[:author_members].each do |author_member|

        journal_paper_params['member_profile_id'] = author_member
        @journal_paper = JournalPaper.new(journal_paper_params)
        @journal_paper.save

        if params[:desktop] == "true"
          render json: {"data" => get_paper_list}.to_json
        end

      end

      redirect_to params['referer_url']

    else

      journal_paper_params['member_profile_id'] = User.find(current_user.id).member_profile_id

      @journal_paper = JournalPaper.new(journal_paper_params)
      @journal_paper.save

      if params[:desktop] == "true"
        render json: {"data" => get_paper_list}.to_json
      end

      redirect_to params['referer_url']

    end

  end

  def edit
    @member = @journal_paper.member_profile
    if params[:desktop]
      render :layout => false
    end
  end

  def update
    @member = @journal_paper.member_profile
    @journal_paper.update_attributes(journal_paper_params)
    @journal_paper.save
    if params[:desktop] == "true"
      render json: {"data" => get_paper_list}.to_json
    else
      redirect_to params['referer_url']
    end
  end

  def destroy
    @journal_paper.destroy
     respond_to do |format|     
      format.html { redirect_to(admin_journal_papers_url) }
      # format.xml  { head :ok }
      format.js
      format.json {render json: {"success" => true}}
    end
  end

  def excel_format
    respond_to do |format|
      format.xlsx {
        response.headers['Content-Disposition'] = 'attachment; filename="journal_paper_format.xlsx"'
      }
    end
  end

  def import_from_excel
    workbook = RubyXL::Parser.parse(params["import_file"].tempfile)
    sheet = workbook[0]
    if sheet.count <= 503
      sheet.each_with_index do |row, i|
        next if i < 3
        user = User.where(:user_name => row.cells[0].value).first rescue nil
        if !user.nil?
          mp = user.member_profile
          import_this_journal_paper(row,mp)
        end
      end
      redirect_to admin_journal_papers_url
    else
      redirect_to admin_journal_papers_url(:error => "1")
    end
  end

  def toggle_hide
    if params[:ids]
      @journal_papers = JournalPaper.any_in(_id: params[:ids])

      @journal_papers.each do |journal_paper|
        journal_paper.is_hidden = params[:disable]
        journal_paper.save
      end
    end

    render json: {"success"=>true}
  end

  def setting
  end


  def frontend_setting
    @member = MemberProfile.find_by(:uid=>params['uid']) rescue nil
    @intro = JournalPaperIntro.find_by(:member_profile_id=>@member.id) rescue nil
    @intro = @intro.nil? ? JournalPaperIntro.new({:member_profile_id=>@member.id}) : @intro
  end

  def update_frontend_setting
    @member = MemberProfile.find(intro_params['member_profile_id']) rescue nil
    @intro = JournalPaperIntro.find_by(:member_profile_id=>@member.id) rescue nil
    @intro = @intro.nil? ? JournalPaperIntro.new({:member_profile_id=>@member.id}) : @intro
    @intro.update_attributes(intro_params)
    @intro.save
    redirect_to URI.encode('/admin/members/'+@member.to_param+'/JournalPaper')
  end

  def get_settings
    @journal_levels = JournalLevel.all
    @author_types = JournalPaperAuthorType.all
    @paper_types = JournalPaperType.all
  end

  def set_plugin
    @plugin = OrbitApp::Plugin::Registration.all.select{|plugin| plugin.app_name.eql? 'JournalPaper'}.first
  end

  private

  def set_journal_paper
    path = request.path.split('/')
    if path.last.include? '-'
      uid = path[-1].split("-").last
      uid = uid.split("?").first
    else
      uid = path[-2].split("-").last
      uid = uid.split("?").first
    end
     @journal_paper = JournalPaper.find_by(:uid => uid) rescue JournalPaper.find(params[:id])
  end

  def journal_paper_params
    params.require(:journal_paper).permit! rescue nil
  end

  def intro_params
    params.require(:journal_paper_intro).permit! rescue nil
  end
end
